@@ -17,12 +17,14 @@ from timely_beliefs.beliefs import utils as belief_utils
 from timely_beliefs.beliefs.utils import is_pandas_structure, is_tb_structure
 from timely_beliefs.db_base import Base
 from timely_beliefs.sensors import utils as sensor_utils
-from timely_beliefs.sensors.classes import DBSensor, Sensor
+from timely_beliefs.sensors.classes import DBSensor, Sensor, SensorDBMixin
 from timely_beliefs.sources import utils as source_utils
 from timely_beliefs.sources.classes import BeliefSource, DBBeliefSource
 from timely_beliefs.visualization import utils as visualization_utils
 
 METADATA = ["sensor", "event_resolution"]
+DatetimeLike = Union[datetime, str, pd.Timestamp]
+TimedeltaLike = Union[timedelta, str, pd.Timedelta]
 
 
 class TimedBelief(object):
@@ -48,22 +50,26 @@ class TimedBelief(object):
     source: BeliefSource
     cumulative_probability: float
 
-    def __init__(
+    def __init__(  # noqa: C901 todo: the noqa can probably be removed when we deprecate the value argument
         self,
         sensor: Sensor,
         source: Union[BeliefSource, str, int],
-        value: float,
+        event_value: Optional[float] = None,
+        value: Optional[float] = None,
         cumulative_probability: Optional[float] = None,
         cp: Optional[float] = None,
         sigma: Optional[float] = None,
-        event_start: Optional[datetime] = None,
-        event_time: Optional[datetime] = None,
-        belief_horizon: Optional[timedelta] = None,
-        belief_time: Optional[datetime] = None,
+        event_start: Optional[DatetimeLike] = None,
+        event_time: Optional[DatetimeLike] = None,
+        belief_horizon: Optional[TimedeltaLike] = None,
+        belief_time: Optional[DatetimeLike] = None,
     ):
         self.sensor = sensor
         self.source = source_utils.ensure_source_exists(source)
-        self.event_value = value
+        # todo: deprecate the 'value' argument in favor of 'event_value' (announced v1.1.0)
+        self.event_value = tb_utils.replace_deprecated_argument(
+            "value", value, "event_value", event_value
+        )
 
         if [cumulative_probability, cp, sigma].count(None) not in (2, 3):
             raise ValueError(
@@ -81,21 +87,21 @@ class TimedBelief(object):
         if [event_start, event_time].count(None) != 1:
             raise ValueError("Must specify either an event_start or an event_time.")
         elif event_start is not None:
-            self.event_start = tb_utils.enforce_tz(event_start, "event_start")
+            self.event_start = tb_utils.parse_datetime_like(event_start, "event_start")
         elif event_time is not None:
             if self.sensor.event_resolution != timedelta():
                 raise KeyError(
                     "Sensor has a non-zero resolution, so it doesn't measure instantaneous events. "
                     "Use event_start instead of event_time."
                 )
-            self.event_start = tb_utils.enforce_tz(event_time, "event_time")
+            self.event_start = tb_utils.parse_datetime_like(event_time, "event_time")
 
         if [belief_horizon, belief_time].count(None) != 1:
             raise ValueError("Must specify either a belief_horizon or a belief_time.")
         elif belief_horizon is not None:
-            self.belief_horizon = belief_horizon
+            self.belief_horizon = tb_utils.parse_timedelta_like(belief_horizon)
         elif belief_time is not None:
-            belief_time = tb_utils.enforce_tz(belief_time, "belief_time")
+            belief_time = tb_utils.parse_datetime_like(belief_time, "belief_time")
             self.belief_horizon = (
                 self.sensor.knowledge_time(self.event_start, self.event_resolution)
                 - belief_time
@@ -161,15 +167,20 @@ class TimedBeliefDBMixin(TimedBelief):
             )
         return None
 
-    event_start = Column(DateTime(timezone=True), primary_key=True)
+    event_start = Column(DateTime(timezone=True), primary_key=True, index=True)
     belief_horizon = Column(Interval(), nullable=False, primary_key=True)
-    cumulative_probability = Column(Float, nullable=False, primary_key=True)
+    cumulative_probability = Column(
+        Float, nullable=False, primary_key=True, default=0.5
+    )
     event_value = Column(Float, nullable=False)
 
     @declared_attr
     def sensor_id(cls):
         return Column(
-            Integer(), ForeignKey("sensor.id", ondelete="CASCADE"), primary_key=True
+            Integer(),
+            ForeignKey("sensor.id", ondelete="CASCADE"),
+            primary_key=True,
+            index=True,
         )
 
     @declared_attr
@@ -180,22 +191,27 @@ class TimedBeliefDBMixin(TimedBelief):
         self,
         sensor: DBSensor,
         source: DBBeliefSource,
-        value: float,
+        event_value: Optional[float] = None,
+        value: Optional[float] = None,
         cumulative_probability: Optional[float] = None,
         cp: Optional[float] = None,
         sigma: Optional[float] = None,
-        event_start: Optional[datetime] = None,
-        event_time: Optional[datetime] = None,
-        belief_horizon: Optional[timedelta] = None,
-        belief_time: Optional[datetime] = None,
+        event_start: Optional[DatetimeLike] = None,
+        event_time: Optional[DatetimeLike] = None,
+        belief_horizon: Optional[TimedeltaLike] = None,
+        belief_time: Optional[DatetimeLike] = None,
     ):
+        # todo: deprecate the 'value' argument in favor of 'event_value' (announced v1.3.0)
+        event_value = tb_utils.replace_deprecated_argument(
+            "value", value, "event_value", event_value
+        )
         self.sensor_id = sensor.id
         self.source_id = source.id
         TimedBelief.__init__(
             self,
             sensor=sensor,
             source=source,
-            value=value,
+            event_value=event_value,
             cumulative_probability=cumulative_probability,
             cp=cp,
             sigma=sigma,
@@ -206,24 +222,63 @@ class TimedBeliefDBMixin(TimedBelief):
         )
 
     @classmethod
-    def query(
+    def add_to_session(
         cls,
         session: Session,
-        sensor: DBSensor,
-        event_before: datetime = None,
-        event_not_before: datetime = None,
-        belief_before: datetime = None,
-        belief_not_before: datetime = None,
-        source: Union[int, List[int], str, List[str]] = None,
+        beliefs_data_frame: "BeliefsDataFrame",
+        commit_transaction: bool = False,
+    ):
+        """Add a BeliefsDataFrame as timed beliefs to a database session.
+
+        :param session: the database session to use
+        :param beliefs_data_frame: the BeliefsDataFrame to be persisted
+        :param commit_transaction: if True, the session is committed
+                                   if False, you can still add other data to the session
+                                   and commit it all within an atomic transaction
+        """
+        # Belief timing is stored as the belief horizon rather than as the belief time
+        belief_records = (
+            beliefs_data_frame.convert_index_from_belief_time_to_horizon()
+            .reset_index()
+            .to_dict("records")
+        )
+        beliefs = [cls(sensor=beliefs_data_frame.sensor, **d) for d in belief_records]
+        session.add_all(beliefs)
+        if commit_transaction:
+            session.commit()
+
+    @classmethod
+    @tb_utils.append_doc_of("TimedBeliefDBMixin.search_session")
+    def query(cls, *args, **kwargs):
+        """Function will be deprecated. Please switch to using search_session."""
+        # todo: deprecate this function (announced v1.3.0), which can clash with SQLAlchemy's Model.query()
+        import warnings
+
+        warnings.warn(
+            "Function 'query' will be replaced by 'search_session'.",
+            FutureWarning,
+        )
+        return cls.search_session(*args, **kwargs)
+
+    @classmethod
+    def search_session(
+        cls,
+        session: Session,
+        sensor: SensorDBMixin,
+        event_before: Optional[datetime] = None,
+        event_not_before: Optional[datetime] = None,
+        belief_before: Optional[datetime] = None,
+        belief_not_before: Optional[datetime] = None,
+        source: Optional[Union[BeliefSource, List[BeliefSource]]] = None,
     ) -> "BeliefsDataFrame":
-        """Query beliefs about sensor events.
+        """Search a database session for beliefs about sensor events.
         :param session: the database session to use
         :param sensor: sensor to which the beliefs pertain
         :param event_before: only return beliefs about events that end before this datetime (inclusive)
         :param event_not_before: only return beliefs about events that start after this datetime (inclusive)
         :param belief_before: only return beliefs formed before this datetime (inclusive)
         :param belief_not_before: only return beliefs formed after this datetime (inclusive)
-        :param source: only return beliefs formed by the given source or list of sources (pass their id or name)
+        :param source: only return beliefs formed by the given source or list of sources
         :returns: a multi-index DataFrame with all relevant beliefs
 
         TODO: rename params for clarity: event_finished_before, event_starts_not_before (or similar), same for beliefs
@@ -231,24 +286,26 @@ class TimedBeliefDBMixin(TimedBelief):
 
         # Check for timezone-aware datetime input
         if event_before is not None:
-            event_before = tb_utils.enforce_tz(event_before, "event_before")
+            event_before = tb_utils.parse_datetime_like(event_before, "event_before")
         if event_not_before is not None:
-            event_not_before = tb_utils.enforce_tz(event_not_before, "event_not_before")
+            event_not_before = tb_utils.parse_datetime_like(
+                event_not_before, "event_not_before"
+            )
         if belief_before is not None:
-            belief_before = tb_utils.enforce_tz(belief_before, "belief_before")
+            belief_before = tb_utils.parse_datetime_like(belief_before, "belief_before")
         if belief_not_before is not None:
-            belief_not_before = tb_utils.enforce_tz(
+            belief_not_before = tb_utils.parse_datetime_like(
                 belief_not_before, "belief_not_before"
             )
 
         # Query sensor for relevant timing properties
         event_resolution, knowledge_horizon_fnc, knowledge_horizon_par = (
             session.query(
-                DBSensor.event_resolution,
-                DBSensor.knowledge_horizon_fnc,
-                DBSensor.knowledge_horizon_par,
+                sensor.__class__.event_resolution,
+                sensor.__class__.knowledge_horizon_fnc,
+                sensor.__class__.knowledge_horizon_par,
             )
-            .filter(DBSensor.id == sensor.id)
+            .filter(sensor.__class__.id == sensor.id)
             .one_or_none()
         )
 
@@ -286,23 +343,9 @@ class TimedBeliefDBMixin(TimedBelief):
 
         # Apply source filter
         if source is not None:
-            source_list = [source] if not isinstance(source, list) else source
-            id_list = [s for s in source_list if isinstance(s, int)]
-            name_list = [s for s in source_list if isinstance(s, str)]
-            if len(id_list) + len(name_list) < len(source_list):
-                unidentifiable_list = [
-                    s
-                    for s in source_list
-                    if not isinstance(s, int) and not isinstance(s, str)
-                ]
-                raise ValueError(
-                    "Query by source failed: query only possible by integer id or string name. Failed sources: %s"
-                    % unidentifiable_list
-                )
-            else:
-                q = q.join(DBBeliefSource).filter(
-                    (cls.source_id.in_(id_list)) | (DBBeliefSource.name.in_(name_list))
-                )
+            sources: list = [source] if not isinstance(source, list) else source
+            source_cls = sources[0].__class__
+            q = q.join(source_cls).filter(cls.source_id.in_([s.id for s in sources]))
 
         # Build our DataFrame of beliefs
         df = BeliefsDataFrame(sensor=sensor, beliefs=q.all())
@@ -342,27 +385,32 @@ class DBTimedBelief(Base, TimedBeliefDBMixin):
         self,
         sensor: DBSensor,
         source: DBBeliefSource,
-        value: float,
+        event_value: Optional[float] = None,
+        value: Optional[float] = None,
         cumulative_probability: Optional[float] = None,
         cp: Optional[float] = None,
         sigma: Optional[float] = None,
-        event_start: Optional[datetime] = None,
-        event_time: Optional[datetime] = None,
-        belief_horizon: Optional[timedelta] = None,
-        belief_time: Optional[datetime] = None,
+        event_start: Optional[DatetimeLike] = None,
+        event_time: Optional[DatetimeLike] = None,
+        belief_horizon: Optional[TimedeltaLike] = None,
+        belief_time: Optional[DatetimeLike] = None,
     ):
+        # todo: deprecate the 'value' argument in favor of 'event_value' (announced v1.3.0)
+        event_value = tb_utils.replace_deprecated_argument(
+            "value", value, "event_value", event_value
+        )
         TimedBeliefDBMixin.__init__(
             self,
-            sensor,
-            source,
-            value,
-            cumulative_probability,
-            cp,
-            sigma,
-            event_start,
-            event_time,
-            belief_horizon,
-            belief_time,
+            sensor=sensor,
+            source=source,
+            event_value=event_value,
+            cumulative_probability=cumulative_probability,
+            cp=cp,
+            sigma=sigma,
+            event_start=event_start,
+            event_time=event_time,
+            belief_horizon=belief_horizon,
+            belief_time=belief_time,
         )
         Base.__init__(self)
 
@@ -419,8 +467,7 @@ class BeliefsDataFrame(pd.DataFrame):
         Method 2:   pass a pandas DataFrame with columns ["event_start", "belief_time", "source", "event_value"]
                     - Optional column: "cumulative_probability" (the default is 0.5)
                     - Alternatively, use keyword arguments to replace columns containing unique values for each belief
-        Method 3:   pass a pandas Series with DatetimeIndex and keyword arguments for "source"
-                    - Optional keyword argument: "belief_horizon" (the default is sensor.event_resolution)
+        Method 3:   pass a pandas Series with DatetimeIndex and keyword arguments for "belief_time" or "belief_horizon", and "source"
                     - Alternatively, use the "event_start" keyword argument to ignore the index
 
     In addition to the standard DataFrame constructor arguments,
@@ -471,9 +518,9 @@ class BeliefsDataFrame(pd.DataFrame):
                 object.__setattr__(self, name, getattr(other, name, None))
         return self
 
-    def __init__(  # noqa: C901
+    def __init__(  # noqa: C901 todo: refactor, e.g. by detecting initialization method
         self, *args, **kwargs
-    ):  # noqa: C901 todo: refactor, e.g. by detecting initialization method
+    ):
         """Initialise a multi-index DataFrame with beliefs about a unique sensor."""
 
         # Initialized with a BeliefsSeries or BeliefsDataFrame
@@ -486,13 +533,13 @@ class BeliefsDataFrame(pd.DataFrame):
 
         # Obtain parameters that are specific to our DataFrame subclass
         sensor: Sensor = kwargs.pop("sensor", None)
-        event_resolution: timedelta = kwargs.pop("event_resolution", None)
+        event_resolution: TimedeltaLike = kwargs.pop("event_resolution", None)
         source: Union[BeliefSource, str, int] = kwargs.pop("source", None)
         source: BeliefSource = source_utils.ensure_source_exists(
             source, allow_none=True
         )
-        event_start: datetime = kwargs.pop("event_start", None)
-        belief_time: datetime = kwargs.pop("belief_time", None)
+        event_start: DatetimeLike = kwargs.pop("event_start", None)
+        belief_time: DatetimeLike = kwargs.pop("belief_time", None)
         belief_horizon: datetime = kwargs.pop("belief_horizon", None)
         cumulative_probability: float = kwargs.pop("cumulative_probability", None)
         beliefs: List[TimedBelief] = kwargs.pop("beliefs", None)
@@ -597,7 +644,7 @@ class BeliefsDataFrame(pd.DataFrame):
                         source_utils.ensure_source_exists
                     )
                 if event_start is not None:
-                    self["event_start"] = tb_utils.enforce_tz(
+                    self["event_start"] = tb_utils.parse_datetime_like(
                         event_start, "event_start"
                     )
                 elif "event_start" not in self and "event_end" not in self:
@@ -606,10 +653,10 @@ class BeliefsDataFrame(pd.DataFrame):
                     )
                 else:
                     self["event_start"] = self["event_start"].apply(
-                        lambda x: tb_utils.enforce_tz(pd.to_datetime(x), "event_start")
+                        lambda x: tb_utils.parse_datetime_like(x, "event_start")
                     )
                 if belief_time is not None:
-                    self["belief_time"] = tb_utils.enforce_tz(
+                    self["belief_time"] = tb_utils.parse_datetime_like(
                         belief_time, "belief_time"
                     )
                 elif belief_horizon is not None:
@@ -620,7 +667,7 @@ class BeliefsDataFrame(pd.DataFrame):
                     )
                 elif "belief_time" in self:
                     self["belief_time"] = self["belief_time"].apply(
-                        lambda x: tb_utils.enforce_tz(pd.to_datetime(x), "belief_time")
+                        lambda x: tb_utils.parse_datetime_like(x, "belief_time")
                     )
                 elif not pd.api.types.is_timedelta64_dtype(
                     self["belief_horizon"]
@@ -656,7 +703,9 @@ class BeliefsDataFrame(pd.DataFrame):
                     ]
                 self.set_index(indices, inplace=True)
 
-        assign_sensor_and_event_resolution(self, sensor, event_resolution)
+        assign_sensor_and_event_resolution(
+            self, sensor, tb_utils.parse_timedelta_like(event_resolution)
+        )
 
     def append_from_time_series(
         self,
@@ -706,7 +755,7 @@ class BeliefsDataFrame(pd.DataFrame):
     @property
     def knowledge_times(self) -> pd.DatetimeIndex:
         return pd.DatetimeIndex(
-            self.event_starts.to_series(name="knowledge_time", keep_tz=True).apply(
+            self.event_starts.to_series(name="knowledge_time").apply(
                 lambda event_start: self.sensor.knowledge_time(
                     event_start, self.event_resolution
                 )
@@ -716,7 +765,7 @@ class BeliefsDataFrame(pd.DataFrame):
     @property
     def knowledge_horizons(self) -> pd.TimedeltaIndex:
         return pd.TimedeltaIndex(
-            self.event_starts.to_series(name="knowledge_horizon", keep_tz=True).apply(
+            self.event_starts.to_series(name="knowledge_horizon").apply(
                 lambda event_start: self.sensor.knowledge_horizon(
                     event_start, self.event_resolution
                 )
@@ -744,7 +793,7 @@ class BeliefsDataFrame(pd.DataFrame):
     @property
     def event_ends(self) -> pd.DatetimeIndex:
         return pd.DatetimeIndex(
-            self.event_starts.to_series(name="event_end", keep_tz=True).apply(
+            self.event_starts.to_series(name="event_end").apply(
                 lambda event_start: event_start + self.event_resolution
             )
         )
@@ -804,12 +853,14 @@ class BeliefsDataFrame(pd.DataFrame):
     @hybrid_method
     def belief_history(
         self,
-        event_start: datetime,
-        belief_time_window: Tuple[Optional[datetime], Optional[datetime]] = (
+        event_start: DatetimeLike,
+        belief_time_window: Tuple[Optional[DatetimeLike], Optional[DatetimeLike]] = (
             None,
             None,
         ),
-        belief_horizon_window: Tuple[Optional[timedelta], Optional[timedelta]] = (
+        belief_horizon_window: Tuple[
+            Optional[TimedeltaLike], Optional[TimedeltaLike]
+        ] = (
             None,
             None,
         ),
@@ -839,14 +890,20 @@ class BeliefsDataFrame(pd.DataFrame):
             return self
 
         df = self.xs(
-            tb_utils.enforce_tz(event_start, "event_start"),
+            tb_utils.parse_datetime_like(event_start, "event_start"),
             level="event_start",
             drop_level=False,
         ).sort_index()
         if belief_time_window[0] is not None:
-            df = df[df.index.get_level_values("belief_time") >= belief_time_window[0]]
+            df = df[
+                df.index.get_level_values("belief_time")
+                >= tb_utils.parse_datetime_like(belief_time_window[0], "belief_time")
+            ]
         if belief_time_window[1] is not None:
-            df = df[df.index.get_level_values("belief_time") <= belief_time_window[1]]
+            df = df[
+                df.index.get_level_values("belief_time")
+                <= tb_utils.parse_datetime_like(belief_time_window[1], "belief_time")
+            ]
         if belief_horizon_window != (None, None):
             if belief_time_window != (None, None):
                 raise ValueError(
@@ -856,12 +913,12 @@ class BeliefsDataFrame(pd.DataFrame):
             if belief_horizon_window[0] is not None:
                 df = df[
                     df.index.get_level_values("belief_horizon")
-                    >= belief_horizon_window[0]
+                    >= tb_utils.parse_timedelta_like(belief_horizon_window[0])
                 ]
             if belief_horizon_window[1] is not None:
                 df = df[
                     df.index.get_level_values("belief_horizon")
-                    <= belief_horizon_window[1]
+                    <= tb_utils.parse_timedelta_like(belief_horizon_window[1])
                 ]
             df = df.convert_index_from_belief_horizon_to_time()
         if not keep_event_start:
@@ -871,8 +928,8 @@ class BeliefsDataFrame(pd.DataFrame):
     @hybrid_method
     def fixed_viewpoint(
         self,
-        belief_time: datetime = None,
-        belief_time_window: Tuple[Optional[datetime], Optional[datetime]] = (
+        belief_time: DatetimeLike = None,
+        belief_time_window: Tuple[Optional[DatetimeLike], Optional[DatetimeLike]] = (
             None,
             None,
         ),
@@ -892,7 +949,7 @@ class BeliefsDataFrame(pd.DataFrame):
         >>> # Select the latest beliefs formed from June 1st to June 6th (up to June 6th 0:00 AM)
         >>> df.fixed_viewpoint(belief_time_window=(datetime(2018, 6, 1, tzinfo=utc), datetime(2018, 6, 6, tzinfo=utc)))
 
-        :param belief_time: datetime indicating the belief should be formed at least before this time
+        :param belief_time: datetime indicating the belief should be formed at least before or at this time
         :param belief_time_window: optional tuple specifying a time window within which beliefs should have been formed
         :param update_belief_times: if True, update the belief time of each belief with the given fixed viewpoint
         """
@@ -912,12 +969,12 @@ class BeliefsDataFrame(pd.DataFrame):
         if belief_time_window[0] is not None:
             df = df[
                 df.index.get_level_values("belief_time")
-                >= tb_utils.enforce_tz(belief_time_window[0], "belief_time")
+                >= tb_utils.parse_datetime_like(belief_time_window[0], "belief_time")
             ]
         if belief_time_window[1] is not None:
             df = df[
                 df.index.get_level_values("belief_time")
-                <= tb_utils.enforce_tz(belief_time_window[1], "belief_time")
+                <= tb_utils.parse_datetime_like(belief_time_window[1], "belief_time")
             ]
         df = belief_utils.select_most_recent_belief(df)
         if update_belief_times is True:
@@ -932,8 +989,10 @@ class BeliefsDataFrame(pd.DataFrame):
     @hybrid_method
     def rolling_viewpoint(
         self,
-        belief_horizon: timedelta = None,
-        belief_horizon_window: Tuple[Optional[timedelta], Optional[timedelta]] = (
+        belief_horizon: TimedeltaLike = None,
+        belief_horizon_window: Tuple[
+            Optional[TimedeltaLike], Optional[TimedeltaLike]
+        ] = (
             None,
             None,
         ),
@@ -974,18 +1033,20 @@ class BeliefsDataFrame(pd.DataFrame):
             df = df.convert_index_from_belief_time_to_horizon()
         if belief_horizon_window[0] is not None:
             df = df[
-                df.index.get_level_values("belief_horizon") >= belief_horizon_window[0]
+                df.index.get_level_values("belief_horizon")
+                >= tb_utils.parse_timedelta_like(belief_horizon_window[0])
             ]
         if belief_horizon_window[1] is not None:
             df = df[
-                df.index.get_level_values("belief_horizon") <= belief_horizon_window[1]
+                df.index.get_level_values("belief_horizon")
+                <= tb_utils.parse_timedelta_like(belief_horizon_window[1])
             ]
         return belief_utils.select_most_recent_belief(df)
 
     @hybrid_method
     def resample_events(
         self,
-        event_resolution: timedelta,
+        event_resolution: TimedeltaLike,
         distribution: Optional[str] = None,
         keep_only_most_recent_belief: bool = False,
     ) -> "BeliefsDataFrame":
@@ -997,6 +1058,7 @@ class BeliefsDataFrame(pd.DataFrame):
 
         if self.empty:
             return self
+        event_resolution = tb_utils.parse_timedelta_like(event_resolution)
         if event_resolution == self.event_resolution:
             return self
         df = self
